
#ifndef VTKFILTERSPARALLELSTATISTICS_EXPORT_H
#define VTKFILTERSPARALLELSTATISTICS_EXPORT_H

#ifdef VTKFILTERSPARALLELSTATISTICS_STATIC_DEFINE
#  define VTKFILTERSPARALLELSTATISTICS_EXPORT
#  define VTKFILTERSPARALLELSTATISTICS_NO_EXPORT
#else
#  ifndef VTKFILTERSPARALLELSTATISTICS_EXPORT
#    ifdef FiltersParallelStatistics_EXPORTS
        /* We are building this library */
#      define VTKFILTERSPARALLELSTATISTICS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKFILTERSPARALLELSTATISTICS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKFILTERSPARALLELSTATISTICS_NO_EXPORT
#    define VTKFILTERSPARALLELSTATISTICS_NO_EXPORT 
#  endif
#endif

#ifndef VTKFILTERSPARALLELSTATISTICS_DEPRECATED
#  define VTKFILTERSPARALLELSTATISTICS_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKFILTERSPARALLELSTATISTICS_DEPRECATED_EXPORT
#  define VTKFILTERSPARALLELSTATISTICS_DEPRECATED_EXPORT VTKFILTERSPARALLELSTATISTICS_EXPORT VTKFILTERSPARALLELSTATISTICS_DEPRECATED
#endif

#ifndef VTKFILTERSPARALLELSTATISTICS_DEPRECATED_NO_EXPORT
#  define VTKFILTERSPARALLELSTATISTICS_DEPRECATED_NO_EXPORT VTKFILTERSPARALLELSTATISTICS_NO_EXPORT VTKFILTERSPARALLELSTATISTICS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKFILTERSPARALLELSTATISTICS_NO_DEPRECATED
#    define VTKFILTERSPARALLELSTATISTICS_NO_DEPRECATED
#  endif
#endif

#endif /* VTKFILTERSPARALLELSTATISTICS_EXPORT_H */
