
#ifndef VTKREMOTINGAPPLICATION_EXPORT_H
#define VTKREMOTINGAPPLICATION_EXPORT_H

#ifdef VTKREMOTINGAPPLICATION_STATIC_DEFINE
#  define VTKREMOTINGAPPLICATION_EXPORT
#  define VTKREMOTINGAPPLICATION_NO_EXPORT
#else
#  ifndef VTKREMOTINGAPPLICATION_EXPORT
#    ifdef RemotingApplication_EXPORTS
        /* We are building this library */
#      define VTKREMOTINGAPPLICATION_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKREMOTINGAPPLICATION_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKREMOTINGAPPLICATION_NO_EXPORT
#    define VTKREMOTINGAPPLICATION_NO_EXPORT 
#  endif
#endif

#ifndef VTKREMOTINGAPPLICATION_DEPRECATED
#  define VTKREMOTINGAPPLICATION_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKREMOTINGAPPLICATION_DEPRECATED_EXPORT
#  define VTKREMOTINGAPPLICATION_DEPRECATED_EXPORT VTKREMOTINGAPPLICATION_EXPORT VTKREMOTINGAPPLICATION_DEPRECATED
#endif

#ifndef VTKREMOTINGAPPLICATION_DEPRECATED_NO_EXPORT
#  define VTKREMOTINGAPPLICATION_DEPRECATED_NO_EXPORT VTKREMOTINGAPPLICATION_NO_EXPORT VTKREMOTINGAPPLICATION_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKREMOTINGAPPLICATION_NO_DEPRECATED
#    define VTKREMOTINGAPPLICATION_NO_DEPRECATED
#  endif
#endif

#endif /* VTKREMOTINGAPPLICATION_EXPORT_H */
