
#ifndef VTKFILTERSPARALLELVERDICT_EXPORT_H
#define VTKFILTERSPARALLELVERDICT_EXPORT_H

#ifdef VTKFILTERSPARALLELVERDICT_STATIC_DEFINE
#  define VTKFILTERSPARALLELVERDICT_EXPORT
#  define VTKFILTERSPARALLELVERDICT_NO_EXPORT
#else
#  ifndef VTKFILTERSPARALLELVERDICT_EXPORT
#    ifdef FiltersParallelVerdict_EXPORTS
        /* We are building this library */
#      define VTKFILTERSPARALLELVERDICT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKFILTERSPARALLELVERDICT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKFILTERSPARALLELVERDICT_NO_EXPORT
#    define VTKFILTERSPARALLELVERDICT_NO_EXPORT 
#  endif
#endif

#ifndef VTKFILTERSPARALLELVERDICT_DEPRECATED
#  define VTKFILTERSPARALLELVERDICT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKFILTERSPARALLELVERDICT_DEPRECATED_EXPORT
#  define VTKFILTERSPARALLELVERDICT_DEPRECATED_EXPORT VTKFILTERSPARALLELVERDICT_EXPORT VTKFILTERSPARALLELVERDICT_DEPRECATED
#endif

#ifndef VTKFILTERSPARALLELVERDICT_DEPRECATED_NO_EXPORT
#  define VTKFILTERSPARALLELVERDICT_DEPRECATED_NO_EXPORT VTKFILTERSPARALLELVERDICT_NO_EXPORT VTKFILTERSPARALLELVERDICT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKFILTERSPARALLELVERDICT_NO_DEPRECATED
#    define VTKFILTERSPARALLELVERDICT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKFILTERSPARALLELVERDICT_EXPORT_H */
