
#ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT_H
#define VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSFILTERSGENERAL_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSGENERAL_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT
#    ifdef VTKExtensionsFiltersGeneral_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_NO_EXPORT
#    define VTKPVVTKEXTENSIONSFILTERSGENERAL_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED
#  define VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERAL_NO_EXPORT VTKPVVTKEXTENSIONSFILTERSGENERAL_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSFILTERSGENERAL_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSFILTERSGENERAL_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSFILTERSGENERAL_EXPORT_H */
