
#ifndef VTKEXPLICITSTRUCTUREDGRID_EXPORT_H
#define VTKEXPLICITSTRUCTUREDGRID_EXPORT_H

#ifdef VTKEXPLICITSTRUCTUREDGRID_STATIC_DEFINE
#  define VTKEXPLICITSTRUCTUREDGRID_EXPORT
#  define VTKEXPLICITSTRUCTUREDGRID_NO_EXPORT
#else
#  ifndef VTKEXPLICITSTRUCTUREDGRID_EXPORT
#    ifdef vtkExplicitStructuredGrid_EXPORTS
        /* We are building this library */
#      define VTKEXPLICITSTRUCTUREDGRID_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKEXPLICITSTRUCTUREDGRID_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKEXPLICITSTRUCTUREDGRID_NO_EXPORT
#    define VTKEXPLICITSTRUCTUREDGRID_NO_EXPORT 
#  endif
#endif

#ifndef VTKEXPLICITSTRUCTUREDGRID_DEPRECATED
#  define VTKEXPLICITSTRUCTUREDGRID_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKEXPLICITSTRUCTUREDGRID_DEPRECATED_EXPORT
#  define VTKEXPLICITSTRUCTUREDGRID_DEPRECATED_EXPORT VTKEXPLICITSTRUCTUREDGRID_EXPORT VTKEXPLICITSTRUCTUREDGRID_DEPRECATED
#endif

#ifndef VTKEXPLICITSTRUCTUREDGRID_DEPRECATED_NO_EXPORT
#  define VTKEXPLICITSTRUCTUREDGRID_DEPRECATED_NO_EXPORT VTKEXPLICITSTRUCTUREDGRID_NO_EXPORT VTKEXPLICITSTRUCTUREDGRID_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKEXPLICITSTRUCTUREDGRID_NO_DEPRECATED
#    define VTKEXPLICITSTRUCTUREDGRID_NO_DEPRECATED
#  endif
#endif

#endif /* VTKEXPLICITSTRUCTUREDGRID_EXPORT_H */
